@@ -19,6 +19,7 @@ def accuracyMetric(target, reported):
     """return Absolute error"""
     return abs(target - reported)
 
+
 def accuracy_metrics(vals, expected_dt, truth_val):
     """
 
@@ -29,25 +30,29 @@ def accuracy_metrics(vals, expected_dt, truth_val):
     """
     accs = [accuracyMetric(truth_val, val) for val in vals[vals.notna()]]
 
+    orig_acc = None
+    avg = None
+    std = None
+    final_acc = None
+
     if len(accs) > 1:
         std = statistics.stdev(accs)
         avg = sum(accs) / len(accs)
-    else:
-        std = None
-        if len(accs) == 1:
-            avg = accs[0]
-        else:
-            avg = None
+        orig_acc = accs[0]
+        final_acc = accs[-1]
+    elif len(accs) == 1:
+        avg = accs[0]
+        orig_acc = accs[0]
+        final_acc = accs[0]
 
     return {
-        "OriginalAccuracy": accs[0],
+        "OriginalAccuracy": orig_acc,
         "AverageAccuracy": avg,
         "AccuracySTDev": std,
-        "FinalAcuracy": accs[-1]
+        "FinalAccuracy": final_acc
     }
 
 
-
 # def completeness(row):
 #     vals = row[row.notna()]
 #     return len(vals) / len(row)
@@ -87,7 +92,6 @@ def time_till_metrics(vals, expected_dt, truth_val):
     }
 
 
-
 # def dq_dimensions_observations(dt, row, final_val):
 #     not_na = row[row.notna()]
 #     if len(not_na) == 0:
@@ -106,52 +110,79 @@ def time_till_metrics(vals, expected_dt, truth_val):
 #     })
 
 
-# def validity_nondecreasing_smoothness(series):
-#     orig_diffs = series.diff()
-#     rises = [abs(diff) for diff in orig_diffs if diff > 0]
-#     inds = [ind for ind, diff in enumerate(orig_diffs) if diff > 0]
-#     if len(inds) == 0:
-#         return 0
-#
-#     runs = [inds[0]]
-#     runs = runs + [abs(ind2 - ind1) for ind1, ind2 in zip(inds[:-1], inds[1:])]
-#     return sum([rise * run / 2 for rise, run in zip(rises, runs)])
+def validity_nondecreasing_smoothness(series):
+    orig_diffs = series.diff()
+    rises = [abs(diff) for diff in orig_diffs if diff > 0]
+    inds = [ind for ind, diff in enumerate(orig_diffs) if diff > 0]
+    if len(inds) == 0:
+        return 0
 
-# def validity_nondecreasing(series):
-#     """Checks if values are nondecreasing.
-#     Return number of times a decrease occurs"""
-#     return len(series[series.diff() < 0])
+    runs = [inds[0]]
+    runs = runs + [abs(ind2 - ind1) for ind1, ind2 in zip(inds[:-1], inds[1:])]
+    return sum([rise * run / 2 for rise, run in zip(rises, runs)])
+
+
+def validity_nondecreasing(series):
+    """Checks if values are nondecreasing.
+    Return number of times a decrease occurs"""
+    return len(series[series.diff() < 0])
 
 
 # ----------------------------------------------#
 # -------- Data Quality Dimensions--------------#
 # ----------------------------------------------#
 
+def apply_to_observations(func):
+    """
+
+    :param func:
+    :return: func
+    """
+
+    def over_observations(data_df, truth_df, expected_df, *args, **kwargs):
+        dct = {}
+
+        for obsv, vals in data_df.iterrows():
+
+            not_na = vals[vals.notna()]
+            if len(not_na) == 0:
+                dct[obsv] = {}
+
+            else:
+                truth_val = truth_df.loc[obsv][0]
+                expected_dt = expected_df.loc[obsv][0]
+
+                dct[obsv] = func(vals, expected_dt, truth_val, *args, **kwargs)
+
+        return pd.DataFrame.from_dict(dct, orient='index')
+
+    return over_observations
+
+
+def apply_to_updates(func):
+    """
+
+    :param func:
+    :return: func
+    """
+
+    def over_updates(data_df, truth_df, expected_df, *args, **kwargs):
+        rot_func = apply_to_observations(func)
+        return rot_func(data_df.T, truth_df.T, expected_df.T, *args, **kwargs)
+
+    return over_updates
+
+
 def timeliness(data_df, truth_df, expected_df):
     """
 
-    :type data_df: object
     :param data_df:
     :param truth_df:
     :param expected_df:
-    :return:pandas dataframe
+    :return:
     """
-
-    dct = {}
-
-    for obsv, vals in data_df.iterrows():
-
-        not_na = vals[vals.notna()]
-        if len(not_na) == 0:
-            dct[obsv] = {}
-
-        else:
-            truth_val = truth_df.loc[obsv][0]
-            expected_dt = expected_df.loc[obsv][0]
-
-            dct[obsv] = time_till_metrics(vals, expected_dt, truth_val)
-
-    return pd.DataFrame.from_dict(dct, orient='index')
+    func = apply_to_observations(time_till_metrics)
+    return func(data_df, truth_df, expected_df)
 
 
 def accuracy(data_df, truth_df, expected_df):
@@ -164,33 +195,30 @@ def accuracy(data_df, truth_df, expected_df):
     :return:pandas dataframe
     """
 
-    dct = {}
-
-    for obsv, vals in data_df.iterrows():
-
-        not_na = vals[vals.notna()]
-        if len(not_na) == 0:
-            dct[obsv] = {}
-
-        else:
-            truth_val = truth_df.loc[obsv][0]
-            expected_dt = expected_df.loc[obsv][0]
-
-            dct[obsv] = accuracy_metrics(vals, expected_dt, truth_val)
-
-    return pd.DataFrame.from_dict(dct, orient='index')
+    func = apply_to_observations(accuracy_metrics)
+    return func(data_df, truth_df, expected_df)
 
 
 def validity(data_df, truth_df, expected_df):
-    pass
+    func = apply_to_updates(validity_nondecreasing_smoothness)
+    df = func(data_df, truth_df, expected_df)
+    func = apply_to_updates(validity_nondecreasing)
+    df2 = func(data_df, truth_df, expected_df)
+    final_df = pd.concat([df, df2], axis=1)
+    return final_df
 
 
-def consistency(data_df, truth_df, expected_df):
-    pass
-
-
-def completness(data_df, truth_df, expected_df):
-    pass
+#
+#
+# def consistency(data_df, truth_df, expected_df):
+#     func = apply_to_observations(accuracy_metrics)
+#     return func(data_df, truth_df, expected_df)
+#
+#
+# def completeness(data_df, truth_df, expected_df):
+#     func1 = apply_to_observations(accuracy_metrics)
+#     func2 = apply_to_updates(accuracy_metrics)
+#     return func(data_df, truth_df, expected_df)
 
 
 # ----------------------------------------------#
@@ -281,7 +309,8 @@ def metric_node(
     """
     funcs_mapping = {
         "accuracy": accuracy,
-        "timeliness": timeliness
+        "timeliness": timeliness,
+        "validity": validity
     }
 
     logging.basicConfig(level=logging.DEBUG,
